#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include <arpa/inet.h>
#include <sys/socket.h>

#include <sys/wait.h>
#include <string.h>

#include <signal.h>
#include <pthread.h>
#include <errno.h>
#include <time.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) { \
			perror(userMsg);\
			exit(EXIT_FAILURE); \
		} \
	} while (0)
	
#define check_pthread(err,userMsg) \
		do {\
			int _err = (err);\
			if (_err > 0) {\
				errno = _err; \
				check_error(0,userMsg);\
			}\
		} while (0)
	
#define PORT (8080)
#define MAX_SIZE (1024)

typedef struct {
	
	int connFd;
} data_t;

int run = 1;

void* threadFunc(void* arg) {
	
	data_t* data = arg;
	
	check_pthread(pthread_detach(pthread_self()),"...");
	
	time_t now = time(NULL);
	char* s = ctime(&now);
	
	int n = strlen(s);
	
	check_error(write(data->connFd, s, n) != -1, "...");
	
	close(data->connFd);
	
	free(data);
	
	return NULL;
}

void processSignal(int signum) {
	
	if (signum == SIGTERM) {
		run = 0;
	}
}	
	
int main(int argc, char** argv){
	
	int listenFd, connFd;
	pthread_t tid;
	
	int opt = 1;
	socklen_t clientLen;
	struct sockaddr_in clientAddr, serverAddr;
	
	listenFd = socket(AF_INET, SOCK_STREAM, 0);
	check_error(listenFd != -1, "socket");
	
	check_error(setsockopt(listenFd, SOL_SOCKET, SO_REUSEADDR | SO_REUSEPORT, &opt, sizeof(opt)) != -1, "...");
	
	memset(&serverAddr, 0, sizeof(struct sockaddr_in));
	serverAddr.sin_family = AF_INET;
	serverAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	serverAddr.sin_port = htons(PORT);
	
	check_error(bind(listenFd, &serverAddr, sizeof(struct sockaddr_in)) != -1, "....");
	
	check_error(listen(listenFd, 5) != -1, "...");
	
	check_error(signal(SIGTERM, processSignal) != SIG_ERR, "...");
	
	while (run) {
		
		clientLen = sizeof(struct sockaddr_in);
		connFd = accept(listenFd, &clientAddr, &clientLen);
		
		check_error(connFd != -1, "...");
		
		data_t* arg = malloc(sizeof(data_t));
		check_error(arg != NULL, "....");
		
		arg->connFd = connFd;
		
		check_pthread(pthread_create(&tid, NULL, threadFunc, arg), "...");
	}
	
	close(listenFd);
	
	exit(EXIT_SUCCESS);
}
