#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include <arpa/inet.h>
#include <sys/socket.h>

#include <sys/wait.h>
#include <string.h>

#include <signal.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) { \
			perror(userMsg);\
			exit(EXIT_FAILURE); \
		} \
	} while (0)
#define PORT (8080)
#define MAX_SIZE (1024)


int run = 1;

void processSignal(int signum) {
	
	if (signum == SIGCHLD) {
		
		check_error(wait(NULL) != -1, "...");
	}
	else if (signum == SIGTERM) {
		run = 0;
	}
}	
	
int main(int argc, char** argv){
	
	int listenFd, connFd;
	pid_t childPid;
	
	int opt = 1;
	socklen_t clientLen;
	struct sockaddr_in clientAddr, serverAddr;
	
	listenFd = socket(AF_INET, SOCK_STREAM, 0);
	check_error(listenFd != -1, "socket");
	
	check_error(setsockopt(listenFd, SOL_SOCKET, SO_REUSEADDR | SO_REUSEPORT, &opt, sizeof(opt)) != -1, "...");
	
	memset(&serverAddr, 0, sizeof(struct sockaddr_in));
	serverAddr.sin_family = AF_INET;
	serverAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	serverAddr.sin_port = htons(PORT);
	
	check_error(bind(listenFd, &serverAddr, sizeof(struct sockaddr_in)) != -1, "....");
	
	check_error(listen(listenFd, 5) != -1, "...");
	
	check_error(signal(SIGCHLD, processSignal) != SIG_ERR, "...");
	check_error(signal(SIGTERM, processSignal) != SIG_ERR, "...");
	
	while (run) {
		
		clientLen = sizeof(struct sockaddr_in);
		connFd = accept(listenFd, &clientAddr, &clientLen);
		
		check_error(connFd != -1, "...");
		
		childPid = fork();
		check_error(childPid != -1, "...");
		
		if (childPid == 0) {
			
			close(listenFd);
			
			char buffer[MAX_SIZE];
			
			int bytesRead = 0;
			while ((bytesRead = read(connFd, buffer, MAX_SIZE)) > 0) {
				
				check_error(write(connFd, buffer, bytesRead) != -1, "...");
			}
			
			check_error(bytesRead != -1, "....");
			
			exit(EXIT_SUCCESS);
		}
		
		close(connFd);
	}
	
	
	close(listenFd);
	
	exit(EXIT_SUCCESS);
}
