#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <pthread.h>
#include <errno.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		} \
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err); \
		if (_err > 0) { \
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

#define STEP (2)	

typedef struct {
	double x;
	double y;
} Tacka_t;

typedef struct {
	int idx;
} arg_t;

Tacka_t* tacke = NULL;
int n = 0;
int capacity = 32;

double min = 0;
int init = 0;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg) {
	
	arg_t* data = arg;
	
	double localMin = 0;
	int lInit = 0;
	double x = tacke[data->idx].x;
	double y = tacke[data->idx].y;
	 
	for (int i = 0; i < n; i++) {
		if (data->idx != i) {
			if (lInit) {
				double r = (tacke[i].x - x)*(tacke[i].x - x) + (tacke[i].y - y)*(tacke[i].y - y);;
				if (r < localMin) {
					localMin = r;
				}
			}
			else {
				localMin = (tacke[i].x - x)*(tacke[i].x - x) + (tacke[i].y - y)*(tacke[i].y - y);
				lInit = 1;
			}
		}
	}
	
	check_pthread(pthread_mutex_lock(&lock), "....");
	if (init) {
		if (localMin < min) {
			min = localMin;
		}
	}
	else {
		min = localMin;
		init = 1;
	}
	check_pthread(pthread_mutex_unlock(&lock), "....");
	
	return NULL;
}

int main(int argc, char** argv){
	
	check_error(argc == 1, "...");
	
	tacke = malloc(sizeof(Tacka_t)*capacity);
	check_error(tacke != NULL, "....");
	
	double x,y;
	while (scanf("%lf%lf", &x, &y) == 2){
		
		tacke[n].x = x;
		tacke[n].y = y;
		n++;
		
		if (n == capacity) {
			
			capacity *= STEP;
			tacke = realloc(tacke, capacity);
			check_error(tacke != NULL, "....");
		}
	}
	
	pthread_t* tids = malloc(n*sizeof(pthread_t));
	check_error(tids != NULL, "....");
	
	arg_t* args = malloc(n*sizeof(arg_t));
	check_error(args != NULL, "....");
	
	for (int i = 0; i < n; i++) {
		args[i].idx = i;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < n; i++) {
		
		check_pthread(pthread_join(tids[i], NULL), "....");
	}
	
	printf("%lf\n", sqrt(min));
	
	free(tacke);
	free(tids);
	free(args);
	check_pthread(pthread_mutex_destroy(&lock), "...");
	
	exit(EXIT_SUCCESS);
}
