#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <pthread.h>
#include <errno.h>

#include <signal.h>
#include <string.h>
#include <ctype.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		} \
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err); \
		if (_err > 0) { \
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)
	
#define MAX_SIZE (257)

int main(int argc, char** argv){
	
	check_error(argc == 4, "...");
	
	int fd = open(argv[1], O_RDWR);
	check_error(fd != -1, "...");
	
	FILE* f = fdopen(fd, "r+");
	check_error(f != NULL, "...");
	
	char rec[MAX_SIZE];
	int failed = 0;
	
	while (fscanf(f, "%s", rec) == 1) {
		
		if (!strcmp(rec, argv[2])) {
			int n = strlen(rec);
			struct flock lock;
			lock.l_type = F_WRLCK;
			lock.l_whence = SEEK_SET;
			lock.l_start = ftell(f);
			lock.l_len = -n;
			
			int retVal = fcntl(fd, F_SETLK, &lock);
			if (retVal == -1) {
				
				if (errno != EACCES && errno != EAGAIN) {
					check_error(0, "...");
				}
				failed++;
			}
			else {
				
				fseek(f, -n, SEEK_CUR);
				fprintf(f, "%s", argv[3]);
				
				lock.l_type = F_UNLCK;
				
				check_error(fcntl(fd, F_SETLK, &lock) != -1, "....");
			}
		}
	}
	
	printf("%d\n", failed);
	
	check_error(feof(f), "....");
	
	fclose(f);
	
	exit(EXIT_SUCCESS);
}
