#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <pthread.h>
#include <errno.h>

#include <signal.h>
#include <string.h>
#include <ctype.h>

#include <sys/mman.h>
#include <semaphore.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		} \
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err); \
		if (_err > 0) { \
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)
	
#define MAX_SIZE (1024)


typedef struct {
	sem_t ready;
	int niz[MAX_SIZE];
	unsigned n;
} data_t;

void* getMemBlock(char* p, unsigned* size) {
	
	int memFd = shm_open(p, O_RDWR, 0600);
	check_error(memFd != -1, "...");
	
	struct stat fInfo;
	check_error(fstat(memFd, &fInfo) != -1, "...");
	*size = fInfo.st_size;
	
	void* addr = mmap(NULL, *size, PROT_READ|PROT_WRITE,MAP_SHARED, memFd, 0);
	check_error(addr != MAP_FAILED, "...");
	
	close(memFd);
	
	return addr;
}

int broj_jedinica(int x) {
	
	unsigned mask = 1<<((sizeof(int)*8) - 1);
	int br = 0;
	
	while (mask) {
		br += x & mask ? 1 : 0;
		mask >>= 1;
	}
	
	return br;
}

int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	unsigned size = 0;
	data_t* memBlock = getMemBlock(argv[1], &size);
	
	check_error(sem_wait(&(memBlock->ready)) != -1, "...");
	
	for (int i = 0; i < memBlock->n; i++) {
		
		if (broj_jedinica(memBlock->niz[i]) >= 4) {
			
			printf("%d\n", memBlock->niz[i]);
		}
	}
	
	check_error(munmap(memBlock, size) != -1, "...");

	exit(EXIT_SUCCESS);
}
