#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <pthread.h>
#include <errno.h>

#include <signal.h>
#include <string.h>
#include <ctype.h>

#include <sys/mman.h>
#include <semaphore.h>

#include <sys/eppol.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		} \
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err); \
		if (_err > 0) { \
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)
	
#define MAX_EVENTS (20)
	
typedef struct {
	int fd;
	FILE* f;
	char* name;
} data_t;


int main(int argc, char** argv){
	
	check_error(argc > 1, "...");

	int nFd = argc - 1;
	
	int epollFd = epoll_create(nFd);
	check_error(epollFd != -1, "....");
	
	struct epoll_event* events = malloc(sizeof(struct epoll_event)*nFd);
	check_error(events != NULL, "...");
	
	for (int i = 0; i < nFd; i++){
		
		int fd = open(argv[i+1], O_RDONLY | O_NONBLOCK);
		check_error(fd != -1, "...");
		
		events[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		
		data_t* data = malloc(sizeof(data_t));
		check_error(data != NULL, "....");
		
		data->fd = fd;
		data->name = argv[i+1];
		data->f = fdopen(fd, "r");
		check_error(data->f != NULL, "...");
		
		events[i].data.ptr = data;
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &events[i]) != -1, "...");
	}
	
	int activeFds = nFd;
	struct epoll_event activeEvents[MAX_EVENTS];
	double zbir = 0;
	
	while (activeFds) {
		
		int retVal = epoll_wait(epollFd, activeEvents, MAX_EVENTS, -1);
		check_error(retVal != -1, "...");
		
		for (int i = 0; i < retVal; i++) {
			
			if (activeEvents[i].events & EPOLLIN) {
				
				data_t* data = activeEvents.data.ptr;
				double x = 0;
				while (fscanf(data->f, "%lf", &x) == 1) {
					zbir += x;
				}
			}
			else if (activeEvents[i].events & (EPOLLERR|EPOLLHUP)) {
				
				data_t* data = activeEvents.data.ptr;
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, data->fd, &activeEvents[i]) != -1, "...");
				
				fclose(data->f);
				free(data);
				activeFds--;
			}
			
		}
		
	}
	
	printf("Zbir je: %lf\n", zbir);
	
	close(epollFd);
	free(events);

	exit(EXIT_SUCCESS);
}
