#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <string.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)){\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err);\
		if (_err > 0) {\
			errno = _err;\
			check_error(0,userMsg);\
		}\
	}while (0)

#define MAX_SIZE 2048

typedef struct {
	int idx;
	int blockSize;
} data_t;
	
char* fPath = NULL;

int globalArray[MAX_SIZE];
int globalCount = 0;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;


void* threadFunc(void* arg) {
	
	data_t* data = arg;
	
	int fd = open(fPath, O_RDONLY);
	check_error(fd != -1, "....");
	
	int start = data->idx*data->blockSize;
	
	check_error(lseek(fd, start, SEEK_SET) != (off_t)-1, "...");
	
	char* buffer = malloc(data->blockSize);
	check_error(buffer != NULL, "...");
	
	check_error(read(fd, buffer, data->blockSize) != -1, "...");
	
	// tag, taa, tga
	
	for (int i = 0; i < data->blockSize - 2; i++) {
		
		if (!strncmp(buffer+i, "taa", 3) || !strncmp(buffer+i, "tag", 3) || !strncmp(buffer+i, "tga", 3)) {
			
			check_pthread(pthread_mutex_lock(&lock), "...");
			globalArray[globalCount] = start + i;
			globalCount++;
			check_pthread(pthread_mutex_unlock(&lock), "...");
		}
	}
	
	close(fd);
	free(buffer);
	
	return NULL;
}
	
int main(int argc, char** argv) {
	
	check_error(argc == 3, "...");
	
	int numOfThreads = atoi(argv[1]);
	fPath = argv[2];
	
	struct stat fInfo;
	check_error(stat(argv[2], &fInfo) != -1, "...");
	
	pthread_t* tids = malloc(numOfThreads*sizeof(pthread_t));
	check_error(tids != NULL, "...");
	
	data_t* args = malloc(numOfThreads*sizeof(data_t));
	check_error(args != NULL, "...");
	
	for (int i = 0; i < numOfThreads; i++) {
		args[i].idx = i;
		args[i].blockSize = fInfo.st_size/numOfThreads;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < numOfThreads; i++) {
		
		check_pthread(pthread_join(tids[i], NULL), "...");
	}
	
	for (int i = 0; i < globalCount; i++) {
		
		printf("%d ", globalArray[i]);
	}
	printf("\n");
	
	free(tids);
	free(args);
	check_pthread(pthread_mutex_destroy(&lock), "....");
	
	exit(EXIT_SUCCESS);
}
