#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <string.h>
#include <sys/wait.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)){\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err);\
		if (_err > 0) {\
			errno = _err;\
			check_error(0,userMsg);\
		}\
	}while (0)

#define MAX_SIZE 256
#define RD_END	(0)
#define WR_END 	(1)
	
int main(int argc, char** argv) {
	
	check_error(argc == 2, "...");
	
	FILE* f = fopen(argv[1], "r");
	check_error(f != NULL, "...");
	
	char* line = NULL;
	size_t lineLen = 0;
	int max = 0;
	char maxCommand[256];
	while (getline(&line, &lineLen, f) != -1) {
		
		int cld2par[2];
		check_error(pipe(cld2par) != -1, "...");
		
		pid_t childPid = fork();
		check_error(childPid != -1, "...");
		
		if (childPid == 0) {
			
			close(cld2par[RD_END]);
			
			check_error(dup2(cld2par[WR_END],STDOUT_FILENO) != -1, "...");
			
			char c1[MAX_SIZE];
			char c2[MAX_SIZE];
			
			sscanf(line, "%s%s", c1, c2);
			
			check_error(execlp(c1, c1, c2, NULL) != -1, "....");			
		}
		
		close(cld2par[WR_END]);
		
		int readBytes = 0;
		int totalBytes = 0;
		char buffer[MAX_SIZE];
		while ((readBytes = read(cld2par[RD_END], buffer, MAX_SIZE)) > 0){
			
			totalBytes += readBytes;
		}
		check_error(readBytes != -1, "...");
		
		if (totalBytes > max) {
			
			max = totalBytes;
			strcpy(maxCommand, line);
		}
		
		check_error(wait(NULL) != -1, "...");
	}
	
	printf("%s", maxCommand);
	
	free(line);	
	fclose(f);
	
	exit(EXIT_SUCCESS);
}
