#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <string.h>
#include <sys/wait.h>

#include <signal.h>
#include <sys/epoll.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)){\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err);\
		if (_err > 0) {\
			errno = _err;\
			check_error(0,userMsg);\
		}\
	}while (0)

#define MAX_SIZE (1024)
#define MAX_EVENTS (20)

typedef struct {
	int fd;
	char* name;
	int bytesRead;
} data_t;

int main(int argc, char** argv) {
	
	check_error(argc > 1, "...");
	
	int numFds = argc - 1;
	
	data_t* data = malloc(sizeof(data_t)*numFds);
	check_error(data != NULL, "...");
	
	struct epoll_event* events = malloc(sizeof(struct epoll_event)*numFds);
	check_error(events != NULL, "...");
	
	int epollFd = epoll_create(numFds);
	check_error(epollFd != -1, "...");
	
	for (int i = 0; i < numFds; i++) {
		
		int fd = open(argv[i+1], O_RDONLY|O_NONBLOCK);
		check_error(fd != -1, "....");
		
		data[i].name = argv[i+1];
		data[i].bytesRead = 0;		
		data[i].fd = fd;
		
		events[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		events[i].data.ptr = &data[i];
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &events[i]) != -1, "...");		
	}
	
	struct epoll_event active[MAX_EVENTS];
	int activeFds = numFds;
	while (activeFds) {
		
		int retVal = epoll_wait(epollFd, active, MAX_EVENTS, -1);
		check_error(retVal != -1, "....");
		
		for (int i = 0; i < retVal; i++) {
			
			if (active[i].events & EPOLLIN) {
				
				data_t* data = active[i].data.ptr;
				int readBytes = 0;
				char buffer[MAX_SIZE];
				while ((readBytes = read(data->fd, buffer, MAX_SIZE)) > 0){
					data->bytesRead += readBytes;
				}
				
				if (readBytes == -1) {
					if (errno != EAGAIN){
						check_error(0, "...");
					}
				}				
			}
			else if (active[i].events & (EPOLLERR | EPOLLHUP)) {
				
				data_t* data = active[i].data.ptr;
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, data->fd, &active[i]) != -1, "...");
				close(data->fd);
				activeFds--;
			}
		}
	}
	
	int min = data[0].bytesRead;
	int min_i = 0;
	for (int i = 1; i<numFds;i++){
		if (data[i].bytesRead < min) {
			min = data[i].bytesRead;
			min_i = i;
		}
	}
	
	char* naziv = strrchr(data[min_i].name, '/');
	printf("%s\n", naziv != NULL ? naziv + 1: data[min_i].name);
	
	close(epollFd);
	free(data);
	free(events);
	
	exit(EXIT_SUCCESS);
}
