#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/epoll.h>
#include <errno.h>
#include <sys/wait.h>

#include <pthread.h>
#include <ctype.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(err,userMsg)\
	do { \
		int _err = (err); \
		if (_err > 0) {\
			errno = _err;\
			check_error(0,userMsg);\
		}\
	} while (0)
		

typedef struct {
	
	int idx;
	char* name;
} data_t;

int histogram[26];
int totalCount = 0;

pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg){
	
	data_t* data = arg;
	
	FILE* f = fopen(data->name, "r");
	check_error(f != NULL, "....");
	
	char c = 0;
	int localCnt = 0;
	while ((c = fgetc(f)) != EOF) {
		
		if (tolower(c) == ('a' + data->idx))
			localCnt++;
	}
	
	histogram[data->idx] = localCnt;
	
	check_pthread(pthread_mutex_lock(&lock), "....");
	totalCount += localCnt;
	check_pthread(pthread_mutex_unlock(&lock), "...");
	
	fclose(f);
	
	return NULL;
}
	
int main(int argc, char** argv) {
	
	check_error(argc == 2, "...");
	
	for (int i = 0; i < 26; i++) {
		histogram[i] = 0;
	}
	
	pthread_t* tids = malloc(26*sizeof(pthread_t));
	check_error(tids != NULL, "...");
	
	data_t* args = malloc(26*sizeof(data_t));
	check_error(args != NULL, "...");
	
	for (int i = 0; i < 26; i++) {
		
		args[i].idx = i;
		args[i].name = argv[1];
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < 26; i++) {
		check_pthread(pthread_join(tids[i], NULL) , "...");
	}
	
	for (int i = 0; i < 26; i++) {
		printf("%d\n", histogram[i]);
	}
	
	printf("%d\n", totalCount);
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
