#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/wait.h>

#include <pthread.h>
#include <errno.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define check_pthread(err,userMsg)\
	do { \
		int _err = (err);\
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	int idx;
} data_t;

int m,n;
double** matrica;

double globalMin;
int init = 0;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void alociraj_matricu(char* path){
	
	FILE* f = fopen(path, "r");
	check_error(f != NULL, "...");
	
	fscanf(f, "%d%d", &m, &n);
	
	matrica = malloc(m*sizeof(double*));
	check_error(matrica != NULL, "...");
	
	for (int i = 0; i < m; i++) {
		matrica[i] = malloc(n*sizeof(double));
		check_error(matrica[i] != NULL, "...");
	}
	
	for (int i = 0; i < m; i++) {
		for (int j = 0; j < n;j ++) {
			
			fscanf(f, "%lf", &matrica[i][j]);
		}
	}
	
	fclose(f);
}

void obrisi_matricu(){
	
	for (int i = 0 ; i < m; i++)
		free(matrica[i]);
		
	free(matrica);
}

void* threadFunc(void* arg) {
	
	data_t* data = arg;
	
	double localMin = matrica[data->idx][0];
	for (int i = 1; i < n; i++) {
		if (matrica[data->idx][i] < localMin) {
			
			localMin = matrica[data->idx][i];
		}
	}
	
	check_pthread(pthread_mutex_lock(&lock), ".....");
	if (init) {
		if (localMin < globalMin) {
			globalMin = localMin;
		}
	}
	else {
		globalMin = localMin;
		init = 1;
	}
	check_pthread(pthread_mutex_unlock(&lock), "...");
	
	return NULL;
}


int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	alociraj_matricu(argv[1]);
	
	pthread_t* tids = malloc(m*sizeof(pthread_t));
	check_error(tids != NULL,"...");
	
	data_t* args = malloc(m*sizeof(data_t));
	check_error(args != NULL, "....");
	
	for (int i = 0; i < m; i++) {
		args[i].idx = i;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < m; i++) {
		
		check_pthread(pthread_join(tids[i], NULL), "...");
	}
	
	printf("%lf\n", globalMin);
	
	obrisi_matricu();
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
