#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/wait.h>

#include <pthread.h>
#include <errno.h>

#include <poll.h>
#include <string.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (256)	

int main(int argc, char** argv){
	
	check_error(argc > 1, "...");
	
	int nFd = argc - 1;
	
	int* bytes = malloc(nFd * sizeof(int));
	check_error(bytes != NULL, "....");
	
	struct pollfd* events = malloc(nFd*sizeof(struct pollfd));
	check_error(events != NULL, "....");
	
	for (int i = 0; i < nFd; i++) {
		
		bytes[i] = 0 ;
		events[i].fd = open(argv[i+1], O_RDONLY|O_NONBLOCK);
		check_error(events[i].fd != -1, "...");
		events[i].events = POLLIN|POLLERR|POLLHUP;
		events[i].revents = 0;
	}
	
	char buffer[MAX_SIZE];
	int activeFds = nFd;
	while (activeFds) {
		
		int retVal = poll(events, nFd, -1);
		check_error(retVal != -1, "...");
		
		for (int i = 0; i < nFd; i++) {
			
			if (events[i].revents & POLLIN) {
				
				int readBytes = 0;
				
				while ((readBytes = read(events[i].fd, buffer, MAX_SIZE)) > 0) {
						bytes[i] += readBytes;
				}
				
				if (readBytes == -1) {
					
					if (errno != EAGAIN) {
						check_error(0, "...");
					}
				}
				
				events[i].revents = 0;
			}
			else if (events[i].revents & (POLLHUP | POLLERR)) {
				
				close(events[i].fd);
				events[i].events = 0;
				events[i].revents = 0;
				events[i].fd = -1;
				
				activeFds--;
			}
		}
	}
	
	int max = bytes[0];
	int imax = 0;
	
	for (int i = 1; i < nFd; i++) {
		if (bytes[i] > max) {
			
			max = bytes[i];
			imax = i;
		}
	}
	
	char* naziv = strrchr(argv[imax+1], '/');
	printf("%s\n", naziv == NULL ? argv[imax+1]:naziv);
	
	free(bytes);
	free(events);
	
	exit(EXIT_SUCCESS);
}
