#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <string.h>

#include <pthread.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define check_pthread(err,userMsg)\
	do {\
		int _err = (err);\
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	int vrsta;
	int kolona;
} data_t;

int** A, **B, **C;
int n, m, m1, k;

int max = 0;
int init = 0;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void ucitaj_matrice(){
	
	scanf("%d%d",&n,&m);
	
	A = malloc(n*sizeof(int*));
	check_error(A != NULL, "...");
	
	for (int i = 0; i < n; i++) {
		A[i] = malloc(m*sizeof(int));
		check_error(A[i] != NULL, "...");
	}
	
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < m; j++) {
			scanf("%d", &A[i][j]);
		}
	}
	
	scanf("%d%d", &m1, &k);	
	
	check_error(m1 == m, "...");
	
	B = malloc(m*sizeof(int*));
	check_error(B != NULL, "...");
	
	for (int i = 0; i < m; i++) {
		B[i] = malloc(k*sizeof(int));
		check_error(B[i] != NULL, "...");
	}
	
	for (int i = 0; i < m; i++) {
		for (int j = 0; j < k; j++) {
			scanf("%d", &B[i][j]);
		}
	}
	
	C = malloc(n*sizeof(int*));
	check_error(C != NULL, "...");
	
	for (int i = 0; i < n; i++) {
		C[i] = malloc(k*sizeof(int));
		check_error(C[i] != NULL, "...");
	}
	
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < k; j++) {
			C[i][j] = 0;
		}
	}
	
}

void oslobodi_matrice(){
	
	for (int i = 0; i < n; i++) {
		free(A[i]);
		free(C[i]);
	}
	
	for (int i = 0; i < m; i++) {
		free(B[i]);
	}
	
	free(A);
	free(B);
	free(C);
}

void* threadFunc(void* arg) {
	
	data_t* data = arg;
	
	for (int i = 0; i < m; i++) {
		C[data->vrsta][data->kolona] += A[data->vrsta][i]*B[i][data->kolona];
	}
	
	check_pthread(pthread_mutex_lock(&lock), "...");
	if (init) {
		if (C[data->vrsta][data->kolona] > max) {
			max = C[data->vrsta][data->kolona];
		}
	}
	else {
		max = C[data->vrsta][data->kolona];
		init = 1;
	}
	check_pthread(pthread_mutex_unlock(&lock), "...");
	
	return NULL;
}

int main(int argc, char** argv){
	
	ucitaj_matrice();

	pthread_t* tids = malloc(n*k*sizeof(pthread_t));
	check_error(tids != NULL, "....");
	
	data_t* args = malloc(n*k*sizeof(data_t));
	check_error(args != NULL, "...");
	
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < k; j++) {
			
			args[i*k + j].vrsta = i;
			args[i*k + j].kolona = j;
			check_pthread(pthread_create(&tids[i*k+j], NULL, threadFunc, &args[i*k+j]), "...");
		}
	}
	
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < k; j++) {
			
			check_pthread(pthread_join(tids[i*k+j], NULL), "...");
		}
	}
	
	
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < k; j++) {
			
			printf("%d ", C[i][j]);
		}
		printf("\n");
	}
	
	
	printf("%d\n", max);
	
	oslobodi_matrice();
	check_pthread(pthread_mutex_destroy(&lock), "...");
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
