#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <string.h>

#include <sys/epoll.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define MAX_SIZE (20)

typedef struct {
	int fd;
	FILE* f;
	char* ime;
	double zbir;
} data_t;

int main(int argc, char** argv){
	
	check_error(argc > 1, "....");
	
	int nFd = argc - 1;
	int epollFd = epoll_create(nFd);
	
	check_error(epollFd != -1, "...");
	
	struct epoll_event* events = malloc(nFd*sizeof(struct epoll_event));
	check_error(events != NULL, "...");
	
	data_t* data = malloc(nFd*sizeof(data_t));
	check_error(data != NULL, "....");
	
	for (int i = 0; i < nFd; i++) {
		
		int fd = open(argv[i+1], O_RDONLY|O_NONBLOCK);
		check_error(fd != -1, "open");
		
		data[i].fd = fd;
		data[i].f = fdopen(fd, "r");
		check_error(data[i].f != NULL, "fdopen");
		
		data[i].zbir = 0;
		data[i].ime = argv[i+1];
		
		events[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		events[i].data.ptr = &data[i];
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &events[i]) != -1, "ctl add");
	}
	
	struct epoll_event active[MAX_SIZE];
	int activeFds = nFd;
	
	while (activeFds) {
		
		int retVal = epoll_wait(epollFd, active, MAX_SIZE, -1);
		check_error(retVal != -1, "wait");
		
		for (int i = 0; i < retVal; i++) {
			
			if (active[i].events & EPOLLIN) {
				
				data_t *arg = active[i].data.ptr;
				double f = 0;
				while (fscanf(arg->f, "%lf", &f) == 1) {
					arg->zbir += f;
				}
			}
			else if (active[i].events & (EPOLLERR|EPOLLHUP)) {
				
				data_t* arg = active[i].data.ptr;
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, arg->fd, &active[i]) != -1, "dtl del");
				
				fclose(arg->f);
				activeFds--;
			}
		}
	}
	
	double max = data[0].zbir;
	int maxI = 0;
	
	for (int i = 1; i < nFd; i++) {
		if (data[i].zbir > max) {
			max = data[i].zbir;
			maxI = i;
		}
	}
	
	printf("%s\n", data[maxI].ime);
	
	close(epollFd);
	free(events);
	free(data);
	
	exit(EXIT_SUCCESS);
}
