#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/epoll.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (20)	

typedef struct {
	
	int fd;
	FILE* f;
	char* ime;
	double zbir;
} data_t;
	
int main(int argc, char** argv){
	
	check_error(argc > 1, "...");
	
	int nFd = argc - 1;
	int epollFd = epoll_create(nFd);
	check_error(epollFd != -1, "...");
	
	struct epoll_event* events = malloc(nFd*sizeof(struct epoll_event));
	check_error(events != NULL, "....");
	
	data_t* args = malloc(nFd*sizeof(data_t));
	check_error(args != NULL, "...");
	
	for (int i = 0; i < nFd; i++) {
		
		int fd = open(argv[i+1], O_RDONLY | O_NONBLOCK);
		check_error(fd != -1, "....");
		
		args[i].fd = fd;
		args[i].f = fdopen(fd, "r");
		check_error(args[i].f != NULL, "...");
		
		args[i].ime = argv[i+1];
		args[i].zbir = 0;
		
		events[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		events[i].data.ptr = &args[i];
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &events[i]) != -1, "...");
	}
	
	struct epoll_event active[MAX_SIZE];
	int activeFds = nFd;
	
	while (activeFds) {
		
		int retVal = epoll_wait(epollFd, active, MAX_SIZE, -1);
		check_error(retVal != -1, "...");
		
		for (int i = 0; i < retVal; i++) {
			
			if (active[i].events & EPOLLIN) {
				
				data_t* arg = active[i].data.ptr;
				
				double x = 0;
				while (fscanf(arg->f, "%lf", &x) == 1) {
					
					arg->zbir += x;
				}
			}
			else if (active[i].events & (EPOLLERR | EPOLLHUP)) {
				
				data_t* arg = active[i].data.ptr;
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, arg->fd, &active[i]) != -1, "...");
				
				fclose(arg->f);
				activeFds--;
			}
		}
	}
	
	double max = args[0].zbir;
	int maxI = 0;
	
	for (int i = 1; i < nFd; i++) {
		if (args[i].zbir > max) {
			
			max = args[i].zbir;
			maxI = i;
		}
	}
	
	printf("%s\n", args[maxI].ime);
	
	free(args);
	free(events);
	
	close(epollFd);
	
	exit(EXIT_SUCCESS);
}
