#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <pthread.h>
#include <errno.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(err,userMsg)\
	do {\
		int _err = (err);\
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	int vrsta;
} data_t;

double** mat;
double p;
int n, m; 

double globalSum = 0;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg) {
	
	data_t* data = arg;
	double localSum = 0;
	
	for (int i = 0; i < m; i++) {
		double x = mat[data->vrsta][i];
		double apsolutnaVrednost = fabs(x);
		double stepen = pow(apsolutnaVrednost, p);
		localSum += stepen;
	}
	
	check_pthread(pthread_mutex_lock(&lock), "...");
	globalSum += localSum;
	check_pthread(pthread_mutex_unlock(&lock), "...");
	
	return NULL;
}
	
int main(int argc, char** argv){
	
	scanf("%lf", &p);
	scanf("%d%d", &n,&m);
	
	mat = malloc(n*sizeof(double*));
	check_error(mat != NULL, "...");
	
	for (int i = 0; i < n; i++) {
		
		mat[i] = malloc(m*sizeof(double));
		check_error(mat[i] != NULL, "...");
	}
	
	for (int i = 0; i < n; i++){
		for (int j = 0; j < m; j++) {
			scanf("%lf", &mat[i][j]);
		}
	}
	
	pthread_t* tids = malloc(n*sizeof(pthread_t));
	check_error(tids != NULL, "...");
	
	data_t* args = malloc(n*sizeof(data_t));
	check_error(args != NULL, "...");
	
	for (int i = 0; i < n; i++) {
		args[i].vrsta = i;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < n; i++) {
		
		check_pthread(pthread_join(tids[i], NULL), "...");
	}
	
	printf("%lf\n", pow(globalSum, 1.0/p));
	
	for (int i = 0; i < n; i++) {
		free(mat[i]);
	}
	free(mat);
	free(tids);
	free(args);
	check_pthread(pthread_mutex_destroy(&lock), "...");
	
	exit(EXIT_SUCCESS);
}

/*
char* path;
int m = 0; // broj niti
int size = 0; // velicina fajla
typedef struct {
	int i;
} data_t;

int globalSum;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg) {
	
	data_t* data = arg;
	
	FILE* f = fopen(path, "r");
	int bytes = size/m;
	
	fseek(f, bytes*data->i, SEEK_SET);
	char c;
	int localSum;
	while ((c = fgetc(f)) && (c != EOF)) {
		if (c == 'w') {
			localSum++;
		}
	}
	
	pthread_mutex_lock(&lock);
	globalSum += localSum;
	pthread_mutex_unlock(&lock);
	 
	fclose(f);
	return NULL;
}
*/
