#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <errno.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)

int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	int fd = open(argv[1], O_RDWR);
	check_error(fd != -1, "....");
	
	FILE* f = fdopen(fd, "r+");
	check_error(f != NULL, "...");
	
	char buffer[MAX_SIZE];
	int success = 0, failed = 0;
	while (fscanf(f, "%s", buffer) == 1) {
		
		struct flock lock;
		lock.l_type = F_WRLCK;
		lock.l_whence = SEEK_SET;
		lock.l_start = ftell(f);
		lock.l_len = -strlen(buffer);
		
		int retVal = fcntl(fd, F_SETLK, &lock);
		if (retVal == -1) {
			if (errno != EAGAIN && errno != EACCES) {
				check_error(0, "...");
			}
			failed++;
		}
		else {
			
			int n = strlen(buffer);
			fseek(f, -n, SEEK_CUR);
			
			for (int i = 0; i < n/2; i++) {
				char c = buffer[n-i-1];
				buffer[n-i-1] = buffer[i];
				buffer[i] = c;
			}
			
			fprintf(f, "%s", buffer);
			success++;
			
			lock.l_type = F_UNLCK;
			check_error(fcntl(fd, F_SETLK, &lock) != -1, "...");
		}
	}
	
	check_error(feof(f), "...");
	
	printf("%d %d\n", success, failed);
	
	fclose(f);
	
	exit(EXIT_SUCCESS);
}

