#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <errno.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <time.h>

#define check_error(expr,userMsg) \
	do {\
		if (!(expr)) {\
			/*fprintf(stderr,"%s\n", userMsg);*/\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while(0)
	
#define MAX_SIZE (1024)

char* getUserName(uid_t ID){
	
	struct passwd* pwd = getpwuid(ID);
	check_error(pwd != NULL, "getpwuid");
	
	char* name = malloc(strlen(pwd->pw_name) + 1);
	check_error(name != NULL, "alokacija");
	strcpy(name, pwd->pw_name);
	
	return name;
}

char* getGroupName(gid_t ID){
	struct group* grp = getgrgid(ID);
	check_error(grp != NULL, "getgrgid");
	
	char* name = malloc(strlen(grp->gr_name) + 1);
	check_error(name != NULL, "alokacija");
	strcpy(name, grp->gr_name);
	
	return name;
}

void fileInfo(char* putanja, char** s) {
	
	errno = EINVAL;
	check_error(*s == NULL, "pokazivac nije inicijalizovan");
	errno = 0;
	
	char* buffer = malloc(MAX_SIZE*sizeof(char));
	check_error(buffer != NULL, "alokacija");
	
	struct stat fInfo;
	check_error(stat(putanja, &fInfo) != -1, "stat");
	
	char fileMode[11];
	strcpy(fileMode, "-rwxrwxrwx");
	
	switch(fInfo.st_mode & S_IFMT) {
		case S_IFREG:
			fileMode[0] = '-';
			break;
		case S_IFDIR:
			fileMode[0] = 'd';
			break;
		case S_IFCHR:
			fileMode[0] = 'c';
			break;
		case S_IFBLK:
			fileMode[0] = 'b';
			break;
		case S_IFIFO:
			fileMode[0] = 'p';
			break;
		case S_IFSOCK:
			fileMode[0] = 's';
			break;
		case S_IFLNK:
			fileMode[0] = 'l';
			break;
	}
	
	//S_ISREG(fInfo.st_mode) == (fInfo.st_mode & S_IFMT == S_IFREG)
	//S_ISDIR(fInfo.st_mode) == (fInfo.st_mode & S_IFMT == S_IFDIR)
	//check_error(S_ISDIR(fInfo.st_mode), "nije dir");
	
	if (!(fInfo.st_mode & S_IRUSR))
		fileMode[1] = '-';
	if (!(fInfo.st_mode & S_IWUSR))
		fileMode[2] = '-';
	if (!(fInfo.st_mode & S_IXUSR))
		fileMode[3] = '-';
	
	if (!(fInfo.st_mode & S_IRGRP))
		fileMode[4] = '-';
	if (!(fInfo.st_mode & S_IWGRP))
		fileMode[5] = '-';
	if (!(fInfo.st_mode & S_IXGRP))
		fileMode[6] = '-';
	
	if (!(fInfo.st_mode & S_IROTH))
		fileMode[7] = '-';
	if (!(fInfo.st_mode & S_IWOTH))
		fileMode[8] = '-';
	if (!(fInfo.st_mode & S_IXOTH))
		fileMode[9] = '-';
	
	int bytes = sprintf(buffer, "%s ", fileMode);
	
	char* userName = getUserName(fInfo.st_uid);
	bytes += sprintf(buffer + bytes, "%s ", userName);
	free(userName);
	
	char* groupName = getGroupName(fInfo.st_gid);
	bytes += sprintf(buffer+bytes, "%s ", groupName);
	free(groupName);
	
	bytes += sprintf(buffer+bytes, "%jd ", (intmax_t)fInfo.st_size);
	
	char* readableTime = ctime(&(fInfo.st_mtime));
	check_error(readableTime != NULL, "...");
	
	bytes += sprintf(buffer+bytes, "%s", readableTime);
	buffer[--bytes] = 0;
	
	bytes += sprintf(buffer+bytes, " %s", putanja);
	
	*s = buffer;	
}
	
int main(int argc, char** argv){
	
	check_error(argc == 2, "argumenti");
	
	char* string = NULL;
	fileInfo(argv[1], &string);
	
	printf("%s\n", string);
	free(string);
		
	exit(EXIT_SUCCESS);
}
