#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#include <pwd.h>
#include <grp.h>

#include <errno.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while(0)

#define MAX_SIZE (1024)

char* getUserName(uid_t ID){
	
	struct passwd* userInfo = getpwuid(ID);
	check_error(userInfo != NULL, "...");
	
	char* userName = malloc(strlen(userInfo->pw_name)+1);
	check_error(userName != NULL, "...");
	strcpy(userName, userInfo->pw_name);
	
	return userName;
}

char* getGroupName(gid_t gid){
		
	struct group* groupInfo = getgrgid(gid);
	check_error(groupInfo != NULL, "...");
	
	char* groupName = malloc(strlen(groupInfo->gr_name) + 1);
	check_error(groupName != NULL, "....");
	strcpy(groupName, groupInfo->gr_name);
	
	return groupName;
}
	
void fileInfo(char* putanja, char** s){
	
	errno = EINVAL;
	check_error(*s == NULL, "argumenti");
	errno = 0;
	
	char* buffer = malloc(MAX_SIZE*sizeof(char));
	check_error(buffer != NULL, "...");
	
	struct stat fInfo;
	check_error(stat(putanja, &fInfo) != -1, "stat");
	
	char pravaPristupa[11];
	strcpy(pravaPristupa, "-rwxrwxrwx");
	
	switch (fInfo.st_mode & S_IFMT){
		case S_IFREG:
			pravaPristupa[0] = '-';
			break;
		case S_IFDIR:
			pravaPristupa[0] = 'd';
			break;
		case S_IFCHR:
			pravaPristupa[0] = 'c';
			break;
		case S_IFBLK:
			pravaPristupa[0] = 'b';
			break;
		case S_IFLNK:
			pravaPristupa[0] = 'l';
			break;
		case S_IFIFO:
			pravaPristupa[0] = 'p';
			break;
		case S_IFSOCK:
			pravaPristupa[0] = 's';
			break;
	}
	/*
	S_ISREG(fInfo.st_mode) <=> (fInfo.st_mode & S_IFMT == S_IFREG)
	
	check_error(S_ISREG(fInfo.st_mode), "....");
	check_error(S_ISDIR(fInfo.st_mode), "....");
	*/
	
	if (!(fInfo.st_mode & S_IRUSR)) 
		pravaPristupa[1] = '-';
	if (!(fInfo.st_mode & S_IWUSR)) 
		pravaPristupa[2] = '-';
	if (!(fInfo.st_mode & S_IXUSR)) 
		pravaPristupa[3] = '-';
		
	if (!(fInfo.st_mode & S_IRGRP)) 
		pravaPristupa[4] = '-';
	if (!(fInfo.st_mode & S_IWGRP)) 
		pravaPristupa[5] = '-';
	if (!(fInfo.st_mode & S_IXGRP)) 
		pravaPristupa[6] = '-';
		
	if (!(fInfo.st_mode & S_IROTH)) 
		pravaPristupa[7] = '-';
	if (!(fInfo.st_mode & S_IWOTH)) 
		pravaPristupa[8] = '-';
	if (!(fInfo.st_mode & S_IXOTH)) 
		pravaPristupa[9] = '-';
		
	int bytes = sprintf(buffer, "%s ", pravaPristupa);
	
	char* userName = getUserName(fInfo.st_uid);
	bytes += sprintf(buffer + bytes, "%s ", userName);
	free(userName);
	
	char* groupName = getGroupName(fInfo.st_gid);
	bytes += sprintf(buffer+ bytes, "%s ", groupName);
	free(groupName);
	
	bytes += sprintf(buffer+ bytes, "%jd ", (intmax_t)fInfo.st_size);
	
	char* vreme = ctime(&(fInfo.st_mtime));
	bytes += sprintf(buffer + bytes, "%s", vreme);
	buffer[--bytes] = 0;
	
	bytes += sprintf(buffer + bytes, " %s", putanja);
	
	*s = buffer;
}	
	
int main(int argc, char** argv){
	
	check_error(argc == 2, "argumenti");

	char* string = NULL;
	fileInfo(argv[1], &string);
	
	printf("%s\n", string);
	free(string);
	
	exit(EXIT_SUCCESS);
}
