#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <errno.h>
#include <time.h>
#include <strings.h>

#include <sys/mman.h>
#include <semaphore.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)

typedef struct{
	
	sem_t semWrite;
	sem_t semRead;
	char buffer[MAX_SIZE];
} sharedType_t;

void* getMemoryBlock(char* fpath, unsigned* size) {
	
	int memFd = shm_open(fpath, O_RDWR|O_CREAT, 0600);
	check_error(memFd != -1, "...");
	
	struct stat fInfo;
	check_error(fstat(memFd, &fInfo) != -1, "....");
	*size = fInfo.st_size;
	
	void* addr = mmap(NULL, *size, PROT_READ|PROT_WRITE, MAP_SHARED, memFd, 0);
	check_error(addr != MAP_FAILED, "...");
	
	close(memFd);
	
	return addr;
}

int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	unsigned size = 0;
	sharedType_t* sharedMem = getMemoryBlock(argv[1], &size);
	
	char buffer[MAX_SIZE];
	do {
		
		check_error(sem_wait(&(sharedMem->semRead)) != -1, "...");
		strcpy(buffer, sharedMem->buffer);
		check_error(sem_post(&(sharedMem->semWrite)) != -1, "...");
		
		printf("Message from writer: %s\n", buffer);

	} while (strcasecmp("quit", buffer));
	
	check_error(munmap(sharedMem, size) != -1, "...");
	
	exit(EXIT_SUCCESS);
}
