#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <errno.h>
#include <time.h>
#include <strings.h>

#include <sys/mman.h>
#include <semaphore.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)

typedef struct{
	
	sem_t semWrite;
	sem_t semRead;
	char buffer[MAX_SIZE];
} sharedType_t;

void* createMemoryBlock(char* fpath, unsigned size) {
	
	int memFd = shm_open(fpath, O_RDWR|O_CREAT, 0600);
	check_error(memFd != -1, "...");
	
	check_error(ftruncate(memFd, size) != -1, "...");
	
	void* addr = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, memFd, 0);
	check_error(addr != MAP_FAILED, "...");
	
	close(memFd);
	
	return addr;
}

int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	sharedType_t* sharedMem = createMemoryBlock(argv[1], sizeof(sharedType_t));
	
	check_error(sem_init(&(sharedMem->semWrite), 1, 1) != -1, "....");
	check_error(sem_init(&(sharedMem->semRead), 1, 0) != -1, "....");
	
	char buffer[MAX_SIZE];
	do {
		printf("Message for reader: \n");
		scanf("%s", buffer);
		
		check_error(sem_wait(&(sharedMem->semWrite)) != -1, "...");
		strcpy(sharedMem->buffer, buffer);
		check_error(sem_post(&(sharedMem->semRead)) != -1, "...");
		
	} while (strcasecmp("quit", buffer));
	
	check_error(munmap(sharedMem, sizeof(sharedType_t)) != -1, "...");
	check_error(shm_unlink(argv[1]) != -1, "...");
	
	exit(EXIT_SUCCESS);
}
