#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <time.h>
#include <strings.h>
#include <string.h>
#include <sys/mman.h>

#include <semaphore.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)

typedef struct {
	
	sem_t semWrite;
	sem_t semRead;
	char buffer[MAX_SIZE];
	
} sharedType_t;

void* getMemBlock(char* path, unsigned* size){
	
	int memFd = shm_open(path, O_RDWR|O_CREAT, 0600);
	check_error(memFd != -1, "...");
	
	struct stat fInfo;
	check_error(fstat(memFd, &fInfo) != -1, "...");
	*size = fInfo.st_size;
	
	void* addr = mmap(NULL, *size, PROT_READ|PROT_WRITE, MAP_SHARED, memFd, 0);
	check_error(addr != MAP_FAILED, "...");
	
	close(memFd);
	
	return addr;
}

int main(int argc, char** argv){
	
	check_error(argc == 2, "....");
	
	unsigned size = 0;
	sharedType_t* shmBlock = getMemBlock(argv[1], &size);
	
	
	char buffer[MAX_SIZE];
	
	do {
		check_error(sem_wait(&(shmBlock->semRead)) != -1, "....");
		strcpy(buffer, shmBlock->buffer);
		check_error(sem_post(&(shmBlock->semWrite)) != -1, "...");
		
		printf("Message from writer: %s\n", buffer);
		
	} while(strcasecmp("quit", buffer));
	
	check_error(munmap(shmBlock, size) != -1, "...");
	
	exit(EXIT_SUCCESS);
}
