#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <sys/mman.h>
#include <semaphore.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (256)

typedef struct {
	
	sem_t semWrite;
	sem_t semRead;
	char buffer[MAX_SIZE];
} sharedData_t;

void* getMemBlock(char* path, unsigned* size){
	
	int memFd = shm_open(path, O_RDWR | O_CREAT, 0600);
	check_error(memFd != -1, "....");
	
	struct stat fInfo;
	check_error(fstat(memFd, &fInfo) != -1, "....");
	*size = fInfo.st_size;
	
	void* addr = mmap(NULL, *size, PROT_READ|PROT_WRITE, MAP_SHARED, memFd, 0);
	check_error(addr != MAP_FAILED, "...");
	
	close(memFd);
	
	return addr;
}

int main(int argc, char** argv) {
	
	check_error(argc == 2, "....");
	
	unsigned size = 0;
	sharedData_t* memBlock = getMemBlock(argv[1], &size);
	
	char buffer[MAX_SIZE];
	
	do {
		
		check_error(sem_wait(&(memBlock->semRead)) != -1, "...");
		strcpy(buffer, memBlock->buffer);
		check_error(sem_post(&(memBlock->semWrite)) != -1, "...");
		
		printf("Message received: %s\n", buffer);
		
	} while (strcasecmp(buffer, "quit"));
	
	check_error(munmap(memBlock, size) != -1, "...");
	
	exit(EXIT_SUCCESS);
}
