#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <sys/mman.h>
#include <semaphore.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (256)

typedef struct {
	
	sem_t semWrite;
	sem_t semRead;
	char buffer[MAX_SIZE];
} sharedData_t;

void* createMemBlock(char* path, unsigned size){
	
	int memFd = shm_open(path, O_RDWR | O_CREAT, 0600);
	check_error(memFd != -1, "....");
	
	check_error(ftruncate(memFd, size) != -1, "...");
	
	void* addr = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, memFd, 0);
	check_error(addr != MAP_FAILED, "...");
	
	close(memFd);
	
	return addr;
}

int main(int argc, char** argv) {
	
	check_error(argc == 2, "....");
	
	sharedData_t* memBlock = createMemBlock(argv[1], sizeof(sharedData_t));
	
	check_error(sem_init(&(memBlock->semWrite), 1, 1) != -1, "...");
	check_error(sem_init(&(memBlock->semRead), 1, 0) != -1, "...");
	
	char buffer[MAX_SIZE];
	
	do {
		
		printf("Message to send or quit:\n");
		scanf("%s", buffer);
		
		check_error(sem_wait(&(memBlock->semWrite)) != -1, "...");
		strcpy(memBlock->buffer, buffer);
		check_error(sem_post(&(memBlock->semRead)) != -1, "...");
		
	} while (strcasecmp(buffer, "quit"));
	
	check_error(munmap(memBlock, sizeof(sharedData_t)) != -1, "...");
	check_error(shm_unlink(argv[1]) != -1, "....");
	
	exit(EXIT_SUCCESS);
}
