#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#define check_error(expr,userMsg)\
	do { \
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

int sigusr1_caught = 0;
int sigusr2_caught = 0;
int shouldTerminate = 0;

void processSignal(int signum) {
	
	switch (signum) {
		
		case SIGUSR1:
			sigusr1_caught = 1;
			break;
		case SIGUSR2:
			sigusr2_caught = 1;
			break;
		case SIGTERM:
			shouldTerminate = 1;
			break;
	}
}
	
int main(int argc, char** argv) {
	
	do {
		check_error(signal(SIGUSR1, processSignal) != SIG_ERR, "....");
		check_error(signal(SIGUSR2, processSignal) != SIG_ERR, "....");
		check_error(signal(SIGTERM, processSignal) != SIG_ERR, "....");
		
		pause();
		
		if (sigusr1_caught == 1) {
			
			printf("SIGUSR1 caught\n");
			sigusr1_caught = 0;
		}
		
		if (sigusr2_caught == 1) {
			
			printf("SIGUSR2 caught\n");
			sigusr2_caught = 0;
		}
		
	} while (!shouldTerminate);
	
	exit(EXIT_SUCCESS);
}
