#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <string.h>

#include <sys/wait.h>
#include <time.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define RD_END (0)
#define WR_END (1)
#define MAX_SIZE (1024)
	
int main(int argc, char** argv) {
	
	int cld2par[2];
	int par2cld[2];
	
	check_error(pipe(cld2par) != -1, "pipe1");
	check_error(pipe(par2cld) != -1, "pipe2");
	
	pid_t childPid = fork();
	check_error(childPid != -1, "fork");
	
	if (childPid > 0) { // parent
		
		close(cld2par[WR_END]);
		close(par2cld[RD_END]);
		
		FILE* f_in = fdopen(cld2par[RD_END], "r");
		check_error(f_in != NULL, "p f in");
		
		FILE* f_out = fdopen(par2cld[WR_END], "w");
		check_error(f_out != NULL, "p  out");
		
		char buffer[MAX_SIZE];
		fprintf(f_out, "Neka poruka: \n");
		fflush(f_out);
		
		check_error(fgets(buffer, MAX_SIZE-1, f_in) != NULL, "p fgets");
		
		printf("%s\n", buffer);
		
		fclose(f_in);
		fclose(f_out);
	}
	else {
		
		close(cld2par[RD_END]);
		close(par2cld[WR_END]);
		
		FILE* f_in = fdopen(par2cld[RD_END], "r");
		check_error(f_in != NULL, "c f in");
		
		FILE* f_out = fdopen(cld2par[WR_END], "w");
		check_error(f_out != NULL, "c f out");
		
		char buffer[MAX_SIZE];
		check_error(fgets(buffer, MAX_SIZE-1, f_in) != NULL, "c fgets");
		
		int n = strlen(buffer);
		buffer[n-2] = 0;
		time_t now = time(NULL);
		strcat(buffer,ctime(&now));
		
		fprintf(f_out, "%s", buffer);
		fflush(f_out);
		
		fclose(f_in);
		fclose(f_out);
		
		exit(EXIT_SUCCESS);
	}
	
	check_error(wait(NULL) != -1, "...");
	
	exit(EXIT_SUCCESS);
}
