#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/epoll.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)
#define MAX_FDS (20)

typedef struct {
	int fd;
	char* name;
} epoll_dt;

// ./1 fifo1 fifo2 .... fifon
int main(int argc, char** argv){
	
	int nFds = argc-1;
	
	int epollFd = epoll_create(nFds);
	check_error(epollFd != -1, "....");
	
	struct epoll_event* fds = malloc(nFds*sizeof(struct epoll_event));
	check_error(fds != NULL, "....");
	
	for (int i = 0; i < nFds; i++) {
		
		int fd = open(argv[i+1], O_RDONLY | O_NONBLOCK);
		check_error(fd != -1, "...");
		
		fds[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		//fds[i].data.fd = fd;
		
		epoll_dt* data = malloc(sizeof(epoll_dt));
		check_error(data != NULL, "....");
		
		data->fd = fd;
		data->name = malloc(strlen(argv[i+1]) + 1);
		check_error(data->name != NULL, "....");
		strcpy(data->name, argv[i+1]);
		
		fds[i].data.ptr = data;
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &fds[i]) != -1, "...");
	}
	
	int activeFds = nFds;
	struct epoll_event activeEvents[MAX_FDS];
	char buffer[MAX_SIZE];
	while (activeFds) {
		
		int numActive = epoll_wait(epollFd, activeEvents, MAX_FDS, -1);
		check_error(numActive != -1, "...");
		
		
		for (int i = 0; i < numActive; i++) {
			
			if (activeEvents[i].events & EPOLLIN) {
				
				epoll_dt* data = (epoll_dt*)activeEvents[i].data.ptr;
				
				printf("Fifo name: %s\n", data->name);
				
				int retVal = read(data->fd, buffer, MAX_SIZE-1);
				check_error(retVal != -1, "...");
				
				buffer[retVal] = 0;
				printf("Message: %s\n", buffer);
				
				/*
				int retVal = read(activeEvents[i].data.fd, buffer, MAX_SIZE-1);
				check_error(retVal != -1, "....");
				
				buffer[retVal] = 0;
				printf("Message: %s\n", buffer);
				*/
			}
			else if (activeEvents[i].events & (EPOLLERR | EPOLLHUP)){
				
				epoll_dt* data = activeEvents[i].data.ptr;
				int fd = data->fd;
				
				//int fd = activeEvents[i].data.fd;
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, fd, &activeEvents[i]) != -1, "...");
				close(fd);
				
				free(data->name);
				free(data);
				
				activeFds--;
			}
		}
	}
	
	free(fds);
	close(epollFd);
	
	exit(EXIT_SUCCESS);
}
