#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <sys/resource.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
typedef struct {
	
	struct timeval wallTime;
	struct timeval userTime;
	struct timeval sysTime;
} stopwatch_t;

void start_stopwatch(stopwatch_t* s){
	
	check_error(gettimeofday(&(s->wallTime), NULL) != -1, "....");
	
	struct rusage usage;
	check_error(getrusage(RUSAGE_SELF, &usage) != -1, "...");
	
	memcpy(&(s->userTime), &(usage.ru_utime), sizeof(struct timeval));
	memcpy(&(s->sysTime), &(usage.ru_stime), sizeof(struct timeval));
}

void print_stopwatch(stopwatch_t* s) {
	
	double ss = s->wallTime.tv_sec  + s->wallTime.tv_usec/1000000.0;
	printf("Wall time: %.6lfs\n", ss);
	
	ss = s->userTime.tv_sec  + s->userTime.tv_usec/1000000.0;
	printf("User time: %.6lfs\n", ss);
	
	ss = s->sysTime.tv_sec  + s->sysTime.tv_usec/1000000.0;
	printf("Sys time: %.6lfs\n", ss);
}

void stop_stopwatch(stopwatch_t* s) {
	
	struct timeval newTime;
	check_error(gettimeofday(&newTime, NULL) != -1, "....");
	
	struct rusage newUsage;
	check_error(getrusage(RUSAGE_SELF, &newUsage) != -1, "...");
	
	int razlika = (newTime.tv_sec * 1000000 + newTime.tv_usec) 
			- (s->wallTime.tv_sec*1000000 + s->wallTime.tv_usec);
			
	s->wallTime.tv_sec = razlika/1000000;
	s->wallTime.tv_usec = razlika%1000000;
	
	razlika = (newUsage.ru_utime.tv_sec * 1000000 + newUsage.ru_utime.tv_usec) 
			- (s->userTime.tv_sec*1000000 + s->userTime.tv_usec);
			
	s->userTime.tv_sec = razlika/1000000;
	s->userTime.tv_usec = razlika%1000000;
	
	razlika = (newUsage.ru_stime.tv_sec * 1000000 + newUsage.ru_stime.tv_usec) 
			- (s->sysTime.tv_sec*1000000 + s->sysTime.tv_usec);
			
	s->sysTime.tv_sec = razlika/1000000;
	s->sysTime.tv_usec = razlika%1000000;
	
}
	
int main(){
	
	stopwatch_t s;
	
	start_stopwatch(&s);
	
	int i = 0;
	double k = 1;
	for (int i = 0; i < 1000000; i++) {
		
		k*=2;
	}
	
	printf("Proizvod: %lf\n", k);
	
	stop_stopwatch(&s);
	
	print_stopwatch(&s);
	
	
	exit(EXIT_SUCCESS);
}
