#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/epoll.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)
#define MAX_EVENTS (20)

typedef struct {
	
	int fd;
	char* name;
} epoll_dt;

// ./1 fifo1 fifo2 .... fifon
int main(int argc, char** argv){
	
	check_error(argc > 1, "....");
	
	int nFds = argc - 1;
	int epollFd = epoll_create(nFds);
	check_error(epollFd != -1, "...");
	
	struct epoll_event* fds = malloc(nFds*sizeof(struct epoll_event));
	check_error(fds != NULL, "...");
	
	for (int i = 0; i < nFds; i++) {
		
		int fd = open(argv[i+1], O_RDONLY | O_NONBLOCK);
		check_error(fd != -1, "....");
		
		fds[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		//fds[i].data.fd = fd;
		
		epoll_dt* tmp = malloc(sizeof(epoll_dt));
		check_error(tmp != NULL, "...");
		
		tmp->fd = fd;
		tmp->name = malloc(strlen(argv[i+1]) + 1);
		check_error(tmp->name != NULL, "...");
		strcpy(tmp->name, argv[i+1]);
		
		fds[i].data.ptr = tmp;
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &fds[i]) != -1, "....");
	}
	
	struct epoll_event activeFds[MAX_EVENTS];
	char buffer[MAX_SIZE];
	int numActiveFds = nFds;
	
	while (numActiveFds) {
		
		int retVal = epoll_wait(epollFd, activeFds, MAX_EVENTS, -1);
		check_error(retVal != -1, "...");
		
		for (int i = 0; i < retVal; i++) {
			
			if (activeFds[i].events & EPOLLIN) {
				
				epoll_dt* tmp = activeFds[i].data.ptr;
				
				int fd = tmp->fd;
				int readBytes = read(fd, buffer, MAX_SIZE-1);
				check_error(readBytes != -1, "....");
				
				buffer[readBytes] = 0;
				printf("Fifo: %s\nMessage: %s\n", tmp->name, buffer);
			}
			else if (activeFds[i].events & (EPOLLERR | EPOLLHUP)) {
				
				epoll_dt* tmp = activeFds[i].data.ptr;
				
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, tmp->fd, &activeFds[i]) != -1, "...");
				close(tmp->fd);
				
				free(tmp->name);
				free(tmp);
				
				numActiveFds--;
			}
		}
	}
	
	
	free(fds);
	close(epollFd);
	
	exit(EXIT_SUCCESS);
}
