#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <poll.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)

// ./1 fifo1 fifo2 .... fifon
int main(int argc, char** argv){
	
	check_error(argc > 1, "....");
	
	int nFds = argc - 1;
	struct pollfd* fds = malloc(nFds*sizeof(struct pollfd));
	check_error(fds != NULL, "...");
	
	for (int i = 0; i < nFds; i++) {
		
		int fd = open(argv[i+1], O_RDONLY | O_NONBLOCK);
		check_error(fd != -1, "...");
		
		fds[i].fd = fd;
		fds[i].events = POLLIN | POLLERR | POLLHUP;
		fds[i].revents = 0;
	}
	
	int activeFds = nFds;
	
	char buffer[MAX_SIZE];
	while (activeFds) {
		
		int retVal = poll(fds, nFds, -1);
		check_error(retVal != -1, "....");
		
		for (int i = 0; i < nFds; i++) {
			
			if (fds[i].revents & POLLIN) {
				
				int readBytes = read(fds[i].fd, buffer, MAX_SIZE -1);
				check_error(readBytes != -1, "....");
				
				buffer[readBytes] = 0;
				printf("Fifo: %s\nMessage: %s\n", argv[i+1], buffer);
				fds[i].revents = 0;
			}
			else if (fds[i].revents & (POLLERR | POLLHUP)) {
				
				close(fds[i].fd);
				fds[i].fd = -1;
				fds[i].events = 0;
				fds[i].revents = 0;
				activeFds--;
			}
		}
		
	}
	
	
	free(fds);
	
	exit(EXIT_SUCCESS);
}
