#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/time.h>
#include <sys/resource.h>
#include <string.h>

#include <sys/epoll.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define MAX_SIZE (1024)
#define MAX_EVENTS (20)

typedef struct {
	
	int fd;
	FILE* f;
	char* ime;
} data_t;

int main(int argc, char** argv){
	
	check_error(argc > 1, "...");
	
	int nFds = argc - 1;
	
	int epollFd = epoll_create(nFds);
	check_error(epollFd != -1, "...");
	
	struct epoll_event* fds = malloc(sizeof(struct epoll_event)*nFds);
	check_error(fds != NULL, "...");
	
	for(int i = 0; i < nFds; i++) {
		
		int fd = open(argv[i+1], O_RDONLY|O_NONBLOCK);
		check_error(fd != -1, "...");
		
		fds[i].events = EPOLLIN|EPOLLERR|EPOLLHUP;
		//fds[i].data.fd = fd;
		
		data_t* d = malloc(sizeof(data_t));
		check_error(d != NULL, "...");
		
		d->fd = fd;
		d->ime = argv[i+1];
		d->f = fdopen(fd, "r");
		check_error(d->f != NULL, "...");
		
		fds[i].data.ptr = d;
		
		check_error(epoll_ctl(epollFd, EPOLL_CTL_ADD, fd, &fds[i]) !=-1,"...");
	}
	
	int activeFds = nFds;
	struct epoll_event active[MAX_EVENTS];
	char buffer[MAX_SIZE];
	
	while (activeFds) {
		
		int retVal = epoll_wait(epollFd, active, MAX_EVENTS, -1);
		check_error(retVal != -1, "...");
		
		for (int i = 0; i < retVal; i++) {
			
			if (active[i].events & EPOLLIN) {
				
				data_t* d = active[i].data.ptr;
				/*
				int bytesRead = read(d->fd, buffer, MAX_SIZE-1);
				check_error(bytesRead != -1, "...");
				
				buffer[bytesRead] = 0;
				*/
				fgets(buffer, MAX_SIZE, d->f);
				printf("%s:\n%s\n", d->ime, buffer);
			}
			else if (active[i].events & (EPOLLERR | EPOLLHUP)){
				
				data_t* d = active[i].data.ptr;
				
				check_error(epoll_ctl(epollFd, EPOLL_CTL_DEL, d->fd, &active[i]) != -1, "...");
				fclose(d->f);
				free(d);
				activeFds--;
			}
		}
	}
	
	
	close(epollFd);
	free(fds);
	
	exit(EXIT_SUCCESS);
}
