#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/time.h>
#include <sys/resource.h>
#include <string.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

typedef struct {
	struct timeval wallTime;
	struct timeval userTime;
	struct timeval sysTime;
} stopwatch_t;

void start(stopwatch_t* s) {
	
	check_error(gettimeofday(&(s->wallTime), NULL) != -1, "...");
	
	struct rusage usage;
	check_error(getrusage(RUSAGE_SELF, &usage) != -1, "...");
	
	memcpy(&(s->userTime),&usage.ru_utime, sizeof(struct timeval));
	memcpy(&(s->sysTime), &usage.ru_stime, sizeof(struct timeval));
}

void stop(stopwatch_t* s){
	
	struct timeval now;
	check_error(gettimeofday(&now, NULL) != -1, "....");
	
	struct rusage usage;
	check_error(getrusage(RUSAGE_SELF, &usage) != -1, "...");
	
	int diff = (now.tv_sec*1000000 + now.tv_usec) 
			- (s->wallTime.tv_sec*1000000 + s->wallTime.tv_usec);
			
	s->wallTime.tv_sec = diff / 1000000;
	s->wallTime.tv_usec = diff % 1000000;
	
	diff = (usage.ru_utime.tv_sec*1000000 + usage.ru_utime.tv_usec) 
			- (s->userTime.tv_sec*1000000 + s->userTime.tv_usec);
			
	s->userTime.tv_sec = diff / 1000000;
	s->userTime.tv_usec = diff % 1000000;
	
	diff = (usage.ru_stime.tv_sec*1000000 + usage.ru_stime.tv_usec) 
			- (s->sysTime.tv_sec*1000000 + s->sysTime.tv_usec);
			
	s->sysTime.tv_sec = diff / 1000000;
	s->sysTime.tv_usec = diff % 1000000;
}

void print(stopwatch_t* s) {
	
	printf("Wall time: %.6lfs\n", s->wallTime.tv_sec + s->wallTime.tv_usec/1000000.0);
	printf("User time: %.6lfs\n", s->userTime.tv_sec + s->userTime.tv_usec/1000000.0);
	printf("Sys time: %.6lfs\n", s->sysTime.tv_sec + s->sysTime.tv_usec/1000000.0);
}

int main(){
	
	stopwatch_t s;
	
	start(&s);
	
	double k = 1;
	for (int i = 0; i < 1000000; i++) {
		k*= i;
	}
	
	printf("%lf\n", k);
	
	stop(&s);
	
	print(&s);
	
	exit(EXIT_SUCCESS);
}
