#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <stdatomic.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define check_pthread(errNum,userMsg)\
	do{\
		int _errNum = (errNum); \
		if (_errNum > 0) {\
			errno = _errNum;\
			check_error(0, userMsg);\
		}\
	} while (0)

#define MAX_SIZE (1024)

typedef struct {
	int idx;
} inArg_t;

int globalCount = 0;
int globalArray[MAX_SIZE];

pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t globalCond = PTHREAD_COND_INITIALIZER;

void* threadFunc(void* arg) {
	
	inArg_t* data = arg;
	
	while (1) {
		
		pthread_mutex_lock(&globalLock);
		
		while (globalCount == 0) {
			
			pthread_cond_wait(&globalCond, &globalLock);
		}
		
		if (globalCount == -1) {
			
			pthread_mutex_unlock(&globalLock);
			break;
		}
		
		printf("Nit: %d\n", data->idx);
		
		int sp = 0;
		
		for (int i = 0; i < globalCount; i++) {
			sp += globalArray[i]*globalArray[i];
		}
		
		printf("Nit: %d\nSkalarni proizvod: %d\n", data->idx, sp);
		
		globalCount = 0;
		
		pthread_mutex_unlock(&globalLock);
	}
	
	printf("Nit %d se zavrsila\n", data->idx);
	
	return NULL;
}

int main(int argc, char** argv) {
		
	check_error(argc == 2, "...");
	
	int numThreads = atoi(argv[1]);
	
	pthread_t* tids = malloc(numThreads*sizeof(pthread_t));
	check_error(tids != NULL, "...");
	
	inArg_t* args = malloc(numThreads*sizeof(inArg_t));
	check_error(args != NULL, "...");
	
	for (int i = 0; i < numThreads; i++) {
		args[i].idx = i+1;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	while (globalCount != -1) {
		
		pthread_mutex_lock(&globalLock);
		
		scanf("%d", &globalCount);
		
		if (globalCount == -1) {
			
			pthread_mutex_unlock(&globalLock);
			pthread_cond_broadcast(&globalCond);
			continue;
		}
		
		for (int i=0; i < globalCount; i++) {
			
			scanf("%d", &globalArray[i]);
		}
		
		
		pthread_mutex_unlock(&globalLock);
		pthread_cond_signal(&globalCond);
	}
	
	for (int i = 0; i < numThreads; i++) {
		
		pthread_join(tids[i], NULL);
	}
	
	pthread_mutex_destroy(&globalLock);
	pthread_cond_destroy(&globalCond);
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
