#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <stdatomic.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(threadErr,userMsg)\
	do {\
		int _err = threadErr;\
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

#define MAX_SIZE (256)

typedef struct {
	
	int idx;
} inArg_t;

int globalSum = 0;
pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t globalCond = PTHREAD_COND_INITIALIZER;

int globalCount = 0;
int globalArray[MAX_SIZE];

void* threadFunc(void* arg) {
	
	inArg_t* data = (inArg_t*)arg;
	
	while (1) {
		
		pthread_mutex_lock(&globalLock);
		
		while (globalCount == 0) {
			
			pthread_cond_wait(&globalCond, &globalLock);
		}
		
		if (globalCount == -1) {
			
			pthread_mutex_unlock(&globalLock);
			break;
		}		
		
		int localSum = 0;
		
		for (int i = 0; i < globalCount; i++) {
			localSum += globalArray[i]*globalArray[i];
		}
		
		printf("Nit %d. Skalarni proizvod: %d\n", data->idx, localSum);
		
		globalCount = 0;
		
		pthread_mutex_unlock(&globalLock);
	}
	
	printf("Nit %d se zavrsila\n", data->idx);
	
	return NULL;
}

int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	int numThreads = atoi(argv[1]);
	pthread_t* tids = malloc(numThreads*sizeof(pthread_t));
	check_error(tids != NULL, "...");
	
	inArg_t* args = malloc(sizeof(inArg_t)*numThreads);
	check_error(args != NULL, "...");
	
	for (int i = 0; i < numThreads; i++) {
		args[i].idx = i+1;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	while (globalCount != -1) {
		
		pthread_mutex_lock(&globalLock);
		
		scanf("%d", &globalCount);
		
		if (globalCount == -1) {
			
			pthread_mutex_unlock(&globalLock);
			pthread_cond_broadcast(&globalCond);
			continue;
		}
		
		for (int i = 0; i < globalCount; i++) {
			
			scanf("%d", &globalArray[i]);
		}
		
		pthread_mutex_unlock(&globalLock);
		
		pthread_cond_signal(&globalCond);
	}
	
	for (int i = 0; i < numThreads; i++) {
		check_pthread(pthread_join(tids[i], NULL), "...");
	}
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
