#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <stdatomic.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define check_pthread(errNum,userMsg)\
	do{\
		int _errNum = (errNum); \
		if (_errNum > 0) {\
			errno = _errNum;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	
	int idx;
	char* putanja;
} inArg_t;

atomic_int globalSum;

void* threadFunc(void* arg) {
	
	inArg_t* data =arg;
	
	FILE* f = fopen(data->putanja, "r");
	check_error(f != NULL, "...");
	
	int localSum = 0;
	
	while (fgetc(f) != EOF)
		localSum++;
	
	atomic_fetch_add(&globalSum, localSum);
	
	fclose(f);
	
	printf("Nit: %d. Karakteri: %d\n", data->idx, localSum);
	
	return NULL;
}

int main(int argc, char** argv) {
	
	check_error(argc > 1, "...");
	
	int numThreads = argc - 1;
	
	pthread_t* tids = malloc(numThreads*sizeof(pthread_t));
	check_error(tids != NULL, "...");
	
	inArg_t* args = malloc(numThreads*sizeof(inArg_t));
	check_error(args != NULL, "...");
	
	atomic_init(&globalSum, 0);
	
	for (int i = 0; i < numThreads; i++) {
		
		args[i].idx = i + 1;
		args[i].putanja = argv[i+1];
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < numThreads; i++) {
		check_pthread(pthread_join(tids[i], NULL), "...");
	}
	
	printf("Ukupno karaktera: %d\n", atomic_load(&globalSum));
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
