#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(threadErr,userMsg)\
	do {\
		int _err = threadErr;\
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	int idx;
} inArg_t;

typedef struct {
	int broj;
} outArg_t;

void* threadFunc(void* arg) {
	
	inArg_t* data = (inArg_t*)arg;
	
	printf("Hello from thread: %d\n", data->idx);
	
	outArg_t* out = malloc(sizeof(outArg_t));
	out->broj = data->idx * data->idx;
	
	return out;
}

int main(int argc, char** argv){
	
	check_error(argc == 2, "...");
	
	int numThreads = atoi(argv[1]);
	pthread_t* tids = malloc(sizeof(pthread_t)*numThreads);
	check_error(tids != NULL, "...");
	
	inArg_t* args = malloc(sizeof(inArg_t)*numThreads);
	check_error(args != NULL, "...");
	
	for (int i = 0; i < numThreads; i++) {
		
		args[i].idx = i+1;
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < numThreads; i++) {
		
		outArg_t* retVal = NULL;
		check_pthread(pthread_join(tids[i], (void**)&retVal), "...");
		
		printf("Return value: %d\n", retVal->broj);
		free(retVal);
	}
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
