#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <stdatomic.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)

#define check_pthread(threadErr,userMsg)\
	do {\
		int _err = threadErr;\
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	
	int idx;
	char* putanja;
} inArg_t;

atomic_int globalSum;

void* threadFunc(void* arg) {
	
	inArg_t* data = (inArg_t*)arg;
	
	FILE* f = fopen(data->putanja, "r");
	check_error(f != NULL, "....");
	
	int cnt = 0;
	
	while (fgetc(f) != EOF)
		cnt++;
		
	atomic_fetch_add(&globalSum, cnt);
	
	fclose(f);
	
	printf("Nit %d: %d\n", data->idx, cnt);
	
	return NULL;
}

int main(int argc, char** argv){
	
	check_error(argc > 1, "...");
	
	atomic_init(&globalSum, 0);
	
	int numThreads = argc - 1;
	pthread_t* tids = malloc(sizeof(pthread_t)*numThreads);
	check_error(tids != NULL, "...");
	inArg_t* args = malloc(sizeof(inArg_t)*numThreads);
	check_error(args != NULL, "...");
	
	for (int i = 0; i < numThreads; i++) {
		
		args[i].idx = i+1;
		args[i].putanja = argv[i+1];
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i < numThreads; i++) {
		check_pthread(pthread_join(tids[i], NULL), "...");
	}
	
	printf("Ukupno ima %d karaktera.\n", atomic_load(&globalSum));
	
	free(tids);
	free(args);
	
	exit(EXIT_SUCCESS);
}
