#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <stdatomic.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define check_pthread(errNum,userMsg)\
	do {\
		int _err = (errNum); \
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

#define MAX_SIZE (256)

typedef struct {
	int idx;
} inArg_t;

int globalArray[MAX_SIZE];
int globalCount = 0;

pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t globalCond = PTHREAD_COND_INITIALIZER;

void* threadFunc(void* arg) {
	
	inArg_t* data = arg;
	
	while (1) {
		
		while (globalCount == 0) {
			pthread_cond_wait(&globalCond, &globalLock);
		}
		
		if (globalCount == -1) {
			
			pthread_mutex_unlock(&globalLock);
			break;
		}
		
		int sp = 0;
		for (int i = 0; i < globalCount; i++) {
			sp += globalArray[i]*globalArray[i];
		}
		
		printf("%d. Skalarni prozivod: %d\n", data->idx, sp);
		
		pthread_mutex_unlock(&globalLock);
	}
	
	printf("Nit %d se zavrsava\n", data->idx);
	
	return NULL;
}

// ./1 +\- imeProm vredn
int main(int argc, char** argv){

	check_error(argc == 2, "...");
	
	int numThreads = atoi(argv[1]);
	
	pthread_t* tids = malloc(sizeof(pthread_t)*numThreads);
	check_error(tids != NULL, "....");
	
	inArg_t* args = malloc(sizeof(inArg_t)*numThreads);
	check_error(args != NULL, "....");
	
	for (int i = 0; i < numThreads; i++) {
		args[i].idx = i+1;
		pthread_create(&tids[i], NULL, threadFunc, &args[i]);
	}
	
	while (globalCount != -1) {
		
		pthread_mutex_lock(&globalLock);
		
		scanf("%d", &globalCount);
		
		if (globalCount == -1) {
			
			pthread_mutex_unlock(&globalLock);
			pthread_cond_broadcast(&globalCond);
			continue;
		}
		
		for (int i = 0; i < globalCount; i++) {
			scanf("%d", &globalArray[i]);
		}
		
		pthread_mutex_unlock(&globalLock);
		
		pthread_cond_signal(&globalCond);
	}
	
	for (int i = 0; i < numThreads; i++) {
		
		pthread_join(tids[i], NULL);
	}
	
	free(args);
	free(tids);
	
	exit(EXIT_SUCCESS);
}
