#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>
#include <errno.h>

#include <stdatomic.h>

#define check_error(expr,userMsg)\
	do {\
		if (!(expr)) {\
			perror(userMsg);\
			exit(EXIT_FAILURE);\
		}\
	} while (0)
	
#define check_pthread(errNum,userMsg)\
	do {\
		int _err = (errNum); \
		if (_err > 0) {\
			errno = _err;\
			check_error(0, userMsg);\
		}\
	} while (0)

typedef struct {
	int idx;
	char* putanja;
} inArg_t;

int globalSum;
pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg) {
	
	inArg_t* data = arg;
	
	FILE* f = fopen(data->putanja, "r");
	check_error(f != NULL, "...");
	
	int localSum = 0;
	
	while (fgetc(f) != EOF)
		localSum++;
		
	pthread_mutex_lock(&globalLock);
	globalSum += localSum;
	pthread_mutex_unlock(&globalLock);
	
	fclose(f);
	
	printf("%d. Karaktera u fajlu: %d\n", data->idx, localSum);
	
	return NULL;
}

// ./1 +\- imeProm vredn
int main(int argc, char** argv){
	
	check_error(argc > 1, "...");
	
	int numThreads = argc - 1;
	pthread_t* tids = malloc(sizeof(pthread_t)*numThreads);
	check_error(tids != NULL, "...");
	
	inArg_t* args = malloc(sizeof(inArg_t)*numThreads);
	check_error(args != NULL, "....");
	
	globalSum = 0;
	
	for (int i = 0; i < numThreads; i++) {
		
		args[i].idx = i+1;
		args[i].putanja = argv[i+1];
		check_pthread(pthread_create(&tids[i], NULL, threadFunc, &args[i]), "...");
	}
	
	for (int i = 0; i<numThreads; i++) {
		
		check_pthread(pthread_join(tids[i], NULL),"...");
	}
	
	printf("Ukupno karaktera: %d\n", globalSum);
	
	free(tids);
	free(args);
	pthread_mutex_destroy(&globalLock);
	
	exit(EXIT_SUCCESS);
}
